import pytest
from rest_framework import status
from history4feed.app.models import Job
from unittest.mock import patch
from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.views import PostOnlyView
from dateutil.parser import parse as parse_date


from history4feed.app.utils import (
    Ordering,
    Pagination,
    MinMaxDateFilter,
)
from dogesec_commons.utils.filters import DatetimeFieldUTC

# from .openapi_params import FEED_PARAMS, POST_PARAMS

from history4feed.app.serializers import PostWithFeedIDSerializer
from history4feed.app.models import Job
from rest_framework import (
    status,
)
from django_filters.rest_framework import (
    DjangoFilterBackend,
)

from tests.utils import Transport


@pytest.mark.parametrize(
    "data",
    [
        dict(title="New title", pubdate="2024-01-01"),
        dict(pubdate="2021-01-01", author="new author 1"),
        dict(categories=['cat1', 'cat2'], author="new author 2"),
        dict(categories=['cat-1', 'cat0'], author="new author 3", pubdate='2025-01-01T10:59:01Z'),
    ]
)
@pytest.mark.django_db
def test_partial_update_post(client, data, feed_posts, api_schema):
    _, (post, _) = feed_posts

    url = f"/api/v1/posts/{post.id}/"

    response = client.patch(url, data, content_type="application/json")

    assert response.status_code == status.HTTP_201_CREATED
    post.refresh_from_db()
    if title := data.get('title'):
        assert post.title == title
    if author := data.get('author'):
        assert post.author == author
    if categories := data.get('categories'):
        assert set([obj.name for obj in post.categories.all()]) == set(categories)
    if pubdate := data.get('pubdate'):
        assert post.pubdate == DatetimeFieldUTC().to_python(parse_date(pubdate))
    api_schema['/api/v1/posts/{post_id}/']['PATCH'].validate_response(Transport.get_st_response(response))
    


@pytest.mark.django_db
def test_destroy_post(client, feed_posts, api_schema):
    _, (post, _) = feed_posts

    url = f"/api/v1/posts/{post.id}/"

    response = client.delete(url)
    assert response.status_code == status.HTTP_204_NO_CONTENT

    post.refresh_from_db()
    assert post.deleted_manually is True

    response = client.get(url)
    assert (
        response.status_code == status.HTTP_404_NOT_FOUND
    ), "should already be deleted"
    api_schema['/api/v1/posts/{post_id}/']['DELETE'].validate_response(Transport.get_st_response(response))


@pytest.mark.django_db
def test_reindex_post(client, feed_posts, api_schema):
    feed, (post, _) = feed_posts

    mock_job = Job.objects.create(state="pending", feed=feed)

    with patch(
        "history4feed.app.views.task_helper.new_patch_posts_job", return_value=mock_job
    ) as mock_task:
        url = f"/api/v1/posts/{post.id}/reindex/"
        data = {}

        response = client.patch(url, data, content_type="application/json")

        assert response.status_code == status.HTTP_201_CREATED
        assert "id" in response.json()
        assert response.json()["id"] == str(mock_job.id)
        assert response.json()["feed_id"] == str(feed.id)

        mock_task.assert_called_once_with(feed, [post])
        api_schema['/api/v1/posts/{post_id}/reindex/']['PATCH'].validate_response(Transport.get_st_response(response))


@pytest.mark.django_db
def test_list_posts(client, feed_posts, api_schema):
    with patch(
        "history4feed.app.views.PostOnlyView.filter_queryset", side_effect=lambda qs: qs
    ) as mock_filter:

        url = "/api/v1/posts/"
        response = client.get(url)

        assert response.status_code == status.HTTP_200_OK
        assert len(response.json()["posts"]) == 2  # Pagination applied
        mock_filter.assert_called_once()
        api_schema['/api/v1/posts/']['GET'].validate_response(Transport.get_st_response(response))


@pytest.mark.django_db
def test_retrieve_post(client, feed_posts, api_schema):
    _, (post, _) = feed_posts

    url = f"/api/v1/posts/{post.id}/"
    response = client.get(url)

    assert response.status_code == status.HTTP_200_OK
    assert response.json()["id"] == str(post.id)
    api_schema['/api/v1/posts/{post_id}/']['GET'].validate_response(Transport.get_st_response(response))



def test_class_variables():
    assert PostOnlyView.ordering_fields == [
        "pubdate",
        "title",
        "datetime_updated",
        "datetime_added",
    ]
    assert PostOnlyView.ordering == "pubdate_descending", "default ordering"
    assert PostOnlyView.filter_backends == [
        DjangoFilterBackend,
        Ordering,
        MinMaxDateFilter,
    ]
    assert type(PostOnlyView.pagination_class) == Pagination
    assert PostOnlyView.pagination_class.results_key == "posts"
    assert PostOnlyView.serializer_class == PostWithFeedIDSerializer



import logging
LOG_PRINT = 105

def newLogger(name: str) -> logging.Logger:
    # Configure logging
    logging.addLevelName(LOG_PRINT, "LOG")
    stream_handler = logging.StreamHandler()  # Log to stdout and stderr
    stream_handler.setLevel(logging.INFO)
    logging.basicConfig(
        level=logging.INFO,
        format=f"%(asctime)s [%(levelname)s] %(message)s",
        handlers=[stream_handler],
        datefmt='%d-%b-%y %H:%M:%S'
    )
    logger = logging.getLogger("history4feed")
    logger.print = lambda msg: logger.log(LOG_PRINT, msg)
    logger.print("=====================history4feed======================")

    return logger

logger = newLogger("h4f-logger")


# Generated by Django 5.0.9 on 2025-02-14 09:30

import django.db.models.deletion
import history4feed.app.models
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('name', history4feed.app.models.SlugField(max_length=1000, primary_key=True, serialize=False)),
            ],
        ),
        migrations.CreateModel(
            name='Feed',
            fields=[
                ('id', models.UUIDField(help_text='UUID of feed generated by history4feed', primary_key=True, serialize=False)),
                ('title', models.CharField(help_text='found in the <channel> of RSS output. Is always kept up to date with the latest feed import values for this property.', max_length=1000)),
                ('description', models.CharField(help_text='found in the <channel> of RSS output. Is always kept up to date with the latest feed import values for this property.', max_length=10240)),
                ('url', models.URLField(help_text='\nThe URL of the RSS or ATOM feed\n\nNote this will be validated to ensure the feed is in the correct format.\n', max_length=1000, unique=True, validators=[history4feed.app.models.normalize_url])),
                ('earliest_item_pubdate', models.DateTimeField(help_text='pubdate of earliest post', null=True)),
                ('latest_item_pubdate', models.DateTimeField(help_text='pubdate of latest post', null=True)),
                ('datetime_added', models.DateTimeField(auto_now_add=True, help_text='date feed entry was added to database')),
                ('feed_type', models.CharField(choices=[('rss', 'Rss'), ('atom', 'Atom'), ('skeleton', 'Skeleton')], editable=False, help_text='type of feed', max_length=12)),
                ('pretty_url', models.URLField(default=None, max_length=1000, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, help_text='UUID of job', primary_key=True, serialize=False)),
                ('state', models.CharField(choices=[('pending', 'Pending'), ('running', 'Running'), ('success', 'Success'), ('failed', 'Failed')], default='pending', help_text='state of the job', max_length=12)),
                ('run_datetime', models.DateTimeField(auto_now_add=True, help_text='time job was executed')),
                ('earliest_item_requested', models.DateTimeField(help_text='shows the earliest time for posts requested. Useful for when jobs are run to see if the time range it runs across is expected', null=True)),
                ('latest_item_requested', models.DateTimeField(help_text='shows the latest time for posts requested', null=True)),
                ('info', models.CharField(help_text='contains a useful summary of the job (e.g. number of posts retrieved, errors logged)', max_length=10240)),
                ('include_remote_blogs', models.BooleanField(default=False)),
                ('feed', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='history4feed.feed')),
            ],
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.UUIDField(help_text='UUID of items generated by history4feed', primary_key=True, serialize=False)),
                ('datetime_added', models.DateTimeField(auto_now_add=True)),
                ('datetime_updated', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(help_text='found in the <item> element of feed output', max_length=1000)),
                ('description', models.CharField(blank=True, help_text='found in the <item> element of feed output', max_length=2097152)),
                ('link', models.URLField(help_text='link to full article. found in the <item> element of feed output', max_length=1000, validators=[history4feed.app.models.normalize_url])),
                ('pubdate', models.DateTimeField(help_text='date of publication.')),
                ('author', models.CharField(blank=True, help_text='author of the post', max_length=1000, null=True)),
                ('is_full_text', models.BooleanField(default=False, help_text='if full text has been retrieved')),
                ('content_type', models.CharField(default='plain/text', help_text='content type of the description', max_length=200)),
                ('added_manually', models.BooleanField(default=False)),
                ('deleted_manually', models.BooleanField(default=False, help_text='this post is hidden from user')),
                ('categories', models.ManyToManyField(blank=True, help_text='categories of the post', related_name='posts', to='history4feed.category')),
                ('feed', models.ForeignKey(help_text='feed id this item belongs too', on_delete=django.db.models.deletion.CASCADE, related_name='posts', to='history4feed.feed')),
            ],
        ),
        migrations.CreateModel(
            name='FulltextJob',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('retrieved', 'Retrieved'), ('skipped', 'Skipped'), ('failed', 'Failed'), ('retrieving', 'Retrieving')], default='retrieving', max_length=15)),
                ('error_str', models.CharField(blank=True, max_length=1500, null=True)),
                ('link', models.CharField(max_length=1500)),
                ('job', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='fulltext_jobs', to='history4feed.job')),
                ('post', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fulltext_jobs', to='history4feed.post')),
            ],
        ),
        migrations.AddConstraint(
            model_name='post',
            constraint=models.UniqueConstraint(fields=('link', 'feed'), name='unique_link_by_feed'),
        ),
    ]



import pytest
from history4feed.h4fscripts.h4f import (
    parse_feed_from_content,
    parse_posts_from_atom_feed,
    parse_posts_from_rss_feed,
)
from .rss_data import rss_example, atom_example


@pytest.mark.parametrize(
    ["data", "url", "expected_feed_data"],
    [
        pytest.param(
            rss_example,
            "https://some_url.net/",
            dict(
                title="Your awesome title",
                description="RSS -- Contains encoded html -- PARTIAL CONTENT ONLY",
                feed_type="rss",
            ),
            id="parse_rss",
        ),
        pytest.param(
            atom_example,
            "https://some_url.net/",
            dict(
                title="Your awesome title",
                description="ATOM -- Contains decoded html inside CDATA tags -- PARTIAL CONTENT ONLY",
                url="",
                feed_type="atom",
            ),
            id="parse_atom",
        ),
    ],
)
def test_parse_feed_from_content(data, url, expected_feed_data):
    feed_data = parse_feed_from_content(data, url)
    for k, v in expected_feed_data.items():
        if v:
            assert feed_data[k] == v


def test_parse_posts_from_rss_feed():
    posts = parse_posts_from_rss_feed("https://example.blog/rss/", rss_example.encode())
    expected_links = [
        "https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html",
        "https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html",
        "https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html",
        "https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html",
        "https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html",
        "https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html",
    ]

    assert expected_links == list(posts)


def test_parse_posts_from_atom_feed():
    posts = parse_posts_from_atom_feed(
        "https://example.blog/rss/", atom_example.encode()
    )
    expected_links = [
        "https://muchdogesec.github.io/fakeblog123///test1/2024/09/01/same-ioc-across-posts.html",
        "https://muchdogesec.github.io/fakeblog123///test1/2024/08/23/obstracts-ai-relationships.html",
        "https://muchdogesec.github.io/fakeblog123///test3/2024/08/20/update-to-post-1.html",
        "https://muchdogesec.github.io/fakeblog123///test2/2024/08/07/testing-extractions-1.html",
        "https://muchdogesec.github.io/fakeblog123///test2/2024/08/05/testing-markdown-elements-1.html",
        "https://muchdogesec.github.io/fakeblog123///test1/2024/08/01/real-post-example-1.html",
    ]

    assert expected_links == list(posts)



import json
import time
from datetime import datetime as dt, UTC
from collections import namedtuple
from urllib.parse import urlencode
from .h4f import FatalError, fetch_page_with_retries
from history4feed.app.settings import history4feed_server_settings as settings
from celery.exceptions import SoftTimeLimitExceeded

DEFAULT_USER_AGENT = "curl"


CDXSearchResult = namedtuple("CDXSearchResult", ["urlkey", "timestamp", "original_url", "mimetype", "statuscode", "digest", "length"])

def cdx_search(url, earliest: dt, latest: dt=None, retry_count=3, sleep_seconds=settings.WAYBACK_SLEEP_SECONDS, user_agent="curl") -> list[CDXSearchResult]:
    latest = latest or dt.now(UTC)
    query = urlencode([
        ("from", as_wayback_date(earliest)),
        ("to", as_wayback_date(latest)),
        ("url", url),
        ("filter", "statuscode:200"),
        ("output", "json"),
        ("collapse", "digest"),
    ])

    headers = {}

    error = None

    for i in range(retry_count+1):
        if i > 0:
            time.sleep(sleep_seconds * 1.5**(i-1))
        try:
            res, content_type, _ = fetch_page_with_retries(f"http://web.archive.org/cdx/search/cdx?{query}", headers=headers)
            res_json = json.loads(res)
            error = None
            break
        except FatalError:
            return []
        except SoftTimeLimitExceeded as e:
            error = e
            break
        except BaseException as e:
            error = e
            continue
    if error:
        raise error
    out = {}
    for v in res_json[1:]:
        try:
            v[6] = int(v[6])
            v[4] = int(v[4])
            v = CDXSearchResult(*v)
            out[v.digest] = v
        except:
            pass
    return list(out.values())
        
def as_wayback_date(date: dt) -> str:
    return date.strftime('%Y%m%d')

def get_wayback_urls(url, from_date, to_date=None):
    to_date = to_date or dt.now(UTC)
    urls = []
    results = cdx_search(url, from_date, to_date)
    for result in results:
        urls.append(f"https://web.archive.org/web/{result.timestamp}id_/{result.original_url}")
    urls.append(url)
    return urls


# Generated by Django 5.2.9 on 2026-03-17 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history4feed', '0009_feed_use_scrapfly_asp'),
    ]

    operations = [
        migrations.AlterField(
            model_name='post',
            name='description',
            field=models.CharField(blank=True, help_text='found in the <item> element of feed output', max_length=4194304),
        ),
    ]
